@@ -17,6 +17,9 @@
 /* MYSQL */
 #undef HAVE_MYSQL
 
+/* PGSQL */
+#undef HAVE_PGSQL
+
 /* DEC */
 #undef HAVE_LL_STRTOUL
 
