@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *      $Id: fttag.c,v 1.15 2003/02/13 02:38:42 maf Exp $
+ *      $Id: fttag.c,v 1.18 2003/12/01 05:48:17 maf Exp $
  */
 
 #include "ftconfig.h"
@@ -72,6 +72,9 @@ struct line_parser {
   char *buf, *fname;
 };
 
+#define FT_TAG_OR_SRCDST (FT_TAG_OR_SRC_TAG|FT_TAG_OR_DST_TAG)
+#define FT_TAG_SET_SRCDST (FT_TAG_SET_SRC_TAG|FT_TAG_SET_DST_TAG)
+
 static int parse_action(struct line_parser *lp, struct fttag *fttag);
 static int parse_action_type(struct line_parser *lp, struct fttag *fttag);
 static int parse_action_match(struct line_parser *lp, struct fttag *fttag);
@@ -115,6 +118,20 @@ static inline void eval_match_tos(struct fttag_action *fta,
   struct fts3rec_v1005 *rec);
 static inline void eval_match_any(struct fttag_action *fta, 
   struct fts3rec_v1005 *rec);
+static inline void eval_match_in_interface(struct fttag_action *fta, 
+  struct fts3rec_v1005 *rec);
+static inline void eval_match_out_interface(struct fttag_action *fta, 
+  struct fts3rec_v1005 *rec);
+static inline void eval_match_interface(struct fttag_action *fta, 
+  struct fts3rec_v1005 *rec);
+static inline void eval_match_exporter(struct fttag_action *fta, 
+  struct fts3rec_v1005 *rec);
+static inline void eval_match_src_ip(struct fttag_action *fta, 
+  struct fts3rec_v1005 *rec);
+static inline void eval_match_dst_ip(struct fttag_action *fta, 
+  struct fts3rec_v1005 *rec);
+static inline void eval_match_ip(struct fttag_action *fta, 
+  struct fts3rec_v1005 *rec);
 
 #define PARSE_STATE_ACTION          0x1
 #define PARSE_STATE_DEFINITION      0x2
@@ -132,7 +149,7 @@ struct jump {
   int (*func)(struct line_parser *lp, struct fttag *fttag);
 };
 
-struct jump pjump[] = {{"tag-action", 0, parse_action},
+static struct jump pjump[] = {{"tag-action", 0, parse_action},
           {"type", PARSE_STATE_ACTION, parse_action_type},
           {"match", PARSE_STATE_ACTION, parse_action_match},
           {"tag-definition", 0, parse_def},
@@ -164,6 +181,9 @@ struct jump pjump[] = {{"tag-action", 0, parse_action},
  * struct fttag_port_look                : port table lookup
  * struct fttag_tos_look                 : tos table lookup
  * struct fttag_next_hop_look            : next hop hash lookup entry
+ * struct fttag_interface_look           : interface table lookup entry
+ * struct fttag_exporter                 : exporter hash lookup entry
+ * struct fttag_ip_look                  : IP address hash lookup entry
  *
  * struct fftag_exp_hash                 : hash table mapping exporter_ip
  *                                         to list of definitions.  Used
@@ -470,48 +490,99 @@ int parse_action_type(struct line_parser *lp, struct fttag *fttag)
   if (!strcasecmp(c, "src-prefix")) {
     lp->cur_action->type = FT_TAG_TYPE_MATCH_SRC_PREFIX;
     lp->cur_action->eval = eval_match_src_prefix;
+  } else if (!strcasecmp(c, "source-prefix")) {
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_SRC_PREFIX;
+      lp->cur_action->eval = eval_match_src_prefix;
   } else if (!strcasecmp(c, "dst-prefix")) {
-    lp->cur_action->type = FT_TAG_TYPE_MATCH_DST_PREFIX;
-    lp->cur_action->eval = eval_match_dst_prefix;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_DST_PREFIX;
+      lp->cur_action->eval = eval_match_dst_prefix;
+  } else if (!strcasecmp(c, "destination-prefix")) {
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_DST_PREFIX;
+      lp->cur_action->eval = eval_match_dst_prefix;
   } else if (!strcasecmp(c, "prefix")) {
-    lp->cur_action->eval = eval_match_prefix;
-    lp->cur_action->type = FT_TAG_TYPE_MATCH_PREFIX;
+      lp->cur_action->eval = eval_match_prefix;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_PREFIX;
   } else if (!strcasecmp(c, "next-hop")) {
-    lp->cur_action->eval = eval_match_nexthop;
-    lp->cur_action->type = FT_TAG_TYPE_MATCH_NEXTHOP;
+      lp->cur_action->eval = eval_match_nexthop;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_NEXTHOP;
   } else if (!strcasecmp(c, "src-as")) {
-    lp->cur_action->type = FT_TAG_TYPE_MATCH_SRC_AS;
-    lp->cur_action->eval = eval_match_src_as;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_SRC_AS;
+      lp->cur_action->eval = eval_match_src_as;
+  } else if (!strcasecmp(c, "source-as")) {
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_SRC_AS;
+      lp->cur_action->eval = eval_match_src_as;
   } else if (!strcasecmp(c, "dst-as")) {
-    lp->cur_action->type = FT_TAG_TYPE_MATCH_DST_AS;
-    lp->cur_action->eval = eval_match_dst_as;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_DST_AS;
+      lp->cur_action->eval = eval_match_dst_as;
+  } else if (!strcasecmp(c, "destination-as")) {
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_DST_AS;
+      lp->cur_action->eval = eval_match_dst_as;
   } else if (!strcasecmp(c, "as")) {
-    lp->cur_action->eval = eval_match_as;
-    lp->cur_action->type = FT_TAG_TYPE_MATCH_AS;
+      lp->cur_action->eval = eval_match_as;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_AS;
   } else if (!strcasecmp(c, "tcp-src-port")) {
-    lp->cur_action->type = FT_TAG_TYPE_MATCH_SRC_TCP_PORT;
-    lp->cur_action->eval = eval_match_tcp_src_port;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_SRC_TCP_PORT;
+      lp->cur_action->eval = eval_match_tcp_src_port;
+  } else if (!strcasecmp(c, "tcp-source-port")) {
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_SRC_TCP_PORT;
+      lp->cur_action->eval = eval_match_tcp_src_port;
   } else if (!strcasecmp(c, "tcp-dst-port")) {
-    lp->cur_action->type = FT_TAG_TYPE_MATCH_DST_TCP_PORT;
-    lp->cur_action->eval = eval_match_tcp_dst_port;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_DST_TCP_PORT;
+      lp->cur_action->eval = eval_match_tcp_dst_port;
+  } else if (!strcasecmp(c, "tcp-destination-port")) {
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_DST_TCP_PORT;
+      lp->cur_action->eval = eval_match_tcp_dst_port;
   } else if (!strcasecmp(c, "tcp-port")) {
-    lp->cur_action->eval = eval_match_tcp_port;
-    lp->cur_action->type = FT_TAG_TYPE_MATCH_TCP_PORT;
+      lp->cur_action->eval = eval_match_tcp_port;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_TCP_PORT;
   } else if (!strcasecmp(c, "udp-src-port")) {
-    lp->cur_action->type = FT_TAG_TYPE_MATCH_SRC_UDP_PORT;
-    lp->cur_action->eval = eval_match_udp_src_port;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_SRC_UDP_PORT;
+      lp->cur_action->eval = eval_match_udp_src_port;
+  } else if (!strcasecmp(c, "udp-source-port")) {
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_SRC_UDP_PORT;
+      lp->cur_action->eval = eval_match_udp_src_port;
   } else if (!strcasecmp(c, "udp-dst-port")) {
-    lp->cur_action->type = FT_TAG_TYPE_MATCH_DST_UDP_PORT;
-    lp->cur_action->eval = eval_match_udp_dst_port;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_DST_UDP_PORT;
+      lp->cur_action->eval = eval_match_udp_dst_port;
+  } else if (!strcasecmp(c, "udp-destination-port")) {
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_DST_UDP_PORT;
+      lp->cur_action->eval = eval_match_udp_dst_port;
   } else if (!strcasecmp(c, "udp-port")) {
-    lp->cur_action->eval = eval_match_udp_port;
-    lp->cur_action->type = FT_TAG_TYPE_MATCH_TCP_PORT;
+      lp->cur_action->eval = eval_match_udp_port;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_TCP_PORT;
   } else if (!strcasecmp(c, "tos")) {
-    lp->cur_action->eval = eval_match_tos;
-    lp->cur_action->type = FT_TAG_TYPE_MATCH_TOS;
+      lp->cur_action->eval = eval_match_tos;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_TOS;
   } else if (!strcasecmp(c, "any")) {
-    lp->cur_action->eval = eval_match_any;
-    lp->cur_action->type = FT_TAG_TYPE_MATCH_ANY;
+      lp->cur_action->eval = eval_match_any;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_ANY;
+  } else if (!strcasecmp(c, "src-ip")) {
+      lp->cur_action->eval = eval_match_src_ip;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_SRC_IP;
+  } else if (!strcasecmp(c, "source-ip-address")) {
+      lp->cur_action->eval = eval_match_src_ip;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_SRC_IP;
+  } else if (!strcasecmp(c, "dst-ip")) {
+      lp->cur_action->eval = eval_match_dst_ip;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_DST_IP;
+  } else if (!strcasecmp(c, "destination-ip-address")) {
+      lp->cur_action->eval = eval_match_dst_ip;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_DST_IP;
+  } else if (!strcasecmp(c, "ip")) {
+      lp->cur_action->eval = eval_match_ip;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_IP;
+  } else if (!strcasecmp(c, "exporter")) {
+      lp->cur_action->eval = eval_match_exporter;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_EXPORTER;
+  } else if (!strcasecmp(c, "input-interface")) {
+      lp->cur_action->eval = eval_match_in_interface;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_IN_INTERFACE;
+  } else if (!strcasecmp(c, "output-interface")) {
+      lp->cur_action->eval = eval_match_out_interface;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_OUT_INTERFACE;
+  } else if (!strcasecmp(c, "interface")) {
+      lp->cur_action->eval = eval_match_interface;
+      lp->cur_action->type = FT_TAG_TYPE_MATCH_INTERFACE;
   } else {
     fterr_warnx("%s line %d: Unrecognized type.", lp->fname, lp->lineno);
     return -1;
@@ -556,20 +627,45 @@ int parse_action_type(struct line_parser *lp, struct fttag *fttag)
 
     bzero(lp->cur_action->look, sizeof (struct fttag_port_look));
 
-  } else if ((lp->cur_action->type & FT_TAG_TYPE_MATCH_PREFIX)) {
+  } else if (lp->cur_action->type & FT_TAG_TYPE_MATCH_PREFIX) {
 
     if (rn_inithead((void**)&lp->cur_action->look, 32) < 0) {
       fterr_warnx("rn_inithead(): failed");
       return -1;
     }
 
-  } else if ((lp->cur_action->type & FT_TAG_TYPE_MATCH_NEXTHOP)) {
+  } else if (lp->cur_action->type & FT_TAG_TYPE_MATCH_NEXTHOP) {
 
     if (!(lp->cur_action->look = ftchash_new(256, sizeof
       (struct fttag_next_hop_look), 4, 16))) {
       fterr_warnx("ftchash_new(): failed");
       return -1;
     }
+
+  } else if (lp->cur_action->type & FT_TAG_TYPE_MATCH_EXPORTER) {
+
+    if (!(lp->cur_action->look = ftchash_new(256, sizeof
+      (struct fttag_exporter_look), 4, 16))) {
+      fterr_warnx("ftchash_new(): failed");
+      return -1;
+    }
+
+  } else if (lp->cur_action->type & FT_TAG_TYPE_MATCH_IP) {
+
+    if (!(lp->cur_action->look = ftchash_new(256, sizeof
+      (struct fttag_ip_look), 4, 16))) {
+      fterr_warnx("ftchash_new(): failed");
+      return -1;
+    }
+
+  } else if (lp->cur_action->type & FT_TAG_TYPE_MATCH_INTERFACE) {
+
+    if (!(lp->cur_action->look = malloc(sizeof (struct fttag_interface_look)))) {
+      fterr_warn("malloc()");
+      return -1;
+    }
+
+    bzero(lp->cur_action->look, sizeof (struct fttag_interface_look));
   }
 
   return 0;
@@ -581,7 +677,7 @@ int parse_action_type(struct line_parser *lp, struct fttag *fttag)
  *
  * process the 'match/set' line.  The match action depends on the type which
  * must be configured first.  An AS match is added to a table, a next-hop
- * is added to a hash (chash_*) and a prefix is added to a radix trie.
+ * is added to a hash (chash_*) and a prefix is added to a radix trie, etc.
  *
  * returns: 0  ok
  *          <0 fail
@@ -594,13 +690,16 @@ int parse_action_match(struct line_parser *lp, struct fttag *fttag)
   struct fttag_prefix_look *prefix_look;
   struct fttag_port_look *port_look;
   struct fttag_tos_look *tos_look;
+  struct fttag_interface_look *interface_look;
+  struct fttag_exporter_look *exporter_look, exporter_look2;
+  struct fttag_ip_look *ip_look, ip_look2;
   struct fttag_any_look *any_look;
   struct ip_prefix ipp;
   struct radix_node_head *rhead;
   u_int32 tag, hash, ipaddr;
-  u_int16 as, port;
+  u_int16 as, port, interface;
   u_int8 tos;
-  int sflag, new;
+  int sflag, new, tflag2, tmpflag, tflag;
   char *c, *match;
 
   if (!lp->cur_action->type) {
@@ -623,6 +722,9 @@ int parse_action_match(struct line_parser *lp, struct fttag *fttag)
   port_look = lp->cur_action->look;
   tos_look = lp->cur_action->look;
   any_look = lp->cur_action->look;
+  interface_look = lp->cur_action->look;
+  ip_look = lp->cur_action->look;
+  exporter_look = lp->cur_action->look;
 
   NEXT_WORD(&lp->buf, c);
 
@@ -636,23 +738,67 @@ int parse_action_match(struct line_parser *lp, struct fttag *fttag)
   NEXT_WORD(&lp->buf, c);
 
   if (!c) {
-    fterr_warnx("%s line %d: Expecting [set|or]-[src|dst].", lp->fname, lp->lineno);
+    fterr_warnx("%s line %d: Expecting [set|or]-[source|destination].",
+    lp->fname, lp->lineno);
     return -1;
   }
 
-  if (!strcasecmp(c, "set-src"))
+  if (!strcasecmp(c, "set-src")) {
     sflag = FT_TAG_SET_SRC_TAG;
-  else if (!strcasecmp(c, "set-dst"))
+    tflag = FT_TAG_SET_SRC_TAG;
+    tflag2 = FT_TAG_OR_SRC_TAG;
+  } else if (!strcasecmp(c, "set-source")) {
+    sflag = FT_TAG_SET_SRC_TAG;
+    tflag = FT_TAG_SET_SRC_TAG;
+    tflag2 = FT_TAG_OR_SRC_TAG;
+  } else if (!strcasecmp(c, "set-dst")) {
     sflag = FT_TAG_SET_DST_TAG;
-  else if (!strcasecmp(c, "or-src"))
+    tflag = FT_TAG_SET_DST_TAG;
+    tflag2 = FT_TAG_OR_DST_TAG;
+  } else if (!strcasecmp(c, "set-destination")) {
+    sflag = FT_TAG_SET_DST_TAG;
+    tflag = FT_TAG_SET_DST_TAG;
+    tflag2 = FT_TAG_OR_DST_TAG;
+  } else if (!strcasecmp(c, "or-src")) {
     sflag = FT_TAG_OR_SRC_TAG;
-  else if (!strcasecmp(c, "or-dst"))
+    tflag = FT_TAG_SET_SRC_TAG;
+    tflag2 = 0;
+  } else if (!strcasecmp(c, "or-source")) {
+    sflag = FT_TAG_OR_SRC_TAG;
+    tflag = FT_TAG_SET_SRC_TAG;
+    tflag2 = 0;
+  } else if (!strcasecmp(c, "or-dst")) {
     sflag = FT_TAG_OR_DST_TAG;
-  else {
-    fterr_warnx("%s line %d: Expecting [set|or]-[src|dst].", lp->fname, lp->lineno);
+    tflag = FT_TAG_SET_DST_TAG;
+    tflag2 = 0;
+  } else if (!strcasecmp(c, "or-destination")) {
+    sflag = FT_TAG_OR_DST_TAG;
+    tflag = FT_TAG_SET_DST_TAG;
+    tflag2 = 0;
+  } else {
+    fterr_warnx("%s line %d: Expecting [set|or]-[source|destination].",
+    lp->fname, lp->lineno);
     return -1;
   }
 
+
+/*
+ * The data structure allows one SET or multiple OR's per
+ * source/destination tag.  Enforce this in the config parser
+ *
+ *         Error conditions
+ *        current     sflag
+ *      ------------------------
+ *       SET_SRC and SET_SRC
+ *       SET_DST and SET_DST
+ *            or
+ *       SET_SRC and OR_SRC
+ *       SET_DST and OR_DST
+ *            or
+ *       OR_SRC and  SET_SRC
+ *       OR_DST and  SET_DST
+*/
+
   NEXT_WORD(&lp->buf, c);
 
   if (!c) {
@@ -666,8 +812,13 @@ int parse_action_match(struct line_parser *lp, struct fttag *fttag)
 
     as = atoi(match);
 
-    if (as_look->set_flags_lookup[as] & sflag) {
-      fterr_warnx("%s line %d: Only one match.", lp->fname, lp->lineno);
+    tmpflag = as_look->set_flags_lookup[as];
+
+    if (((tmpflag & FT_TAG_SET_SRCDST) & tflag) ||
+       ((tmpflag & FT_TAG_OR_SRCDST) & tflag2)) {
+      fterr_warnx(
+        "%s line %d: Only one set per source/destination per match.",
+        lp->fname, lp->lineno);
       return -1;
     }
 
@@ -681,17 +832,51 @@ int parse_action_match(struct line_parser *lp, struct fttag *fttag)
 
     if (sflag & FT_TAG_SET_DST_TAG)
       as_look->dst_tag_lookup[as] = tag;
-
+  
     if (sflag & FT_TAG_OR_DST_TAG)
       as_look->dst_tag_lookup[as] |= tag;
 
+  } else if (lp->cur_action->type & FT_TAG_TYPE_MATCH_INTERFACE) {
+
+    interface = atoi(match);
+
+    tmpflag = interface_look->set_flags_lookup[interface];
+
+    if (((tmpflag & FT_TAG_SET_SRCDST) & tflag) ||
+       ((tmpflag & FT_TAG_OR_SRCDST) & tflag2)) {
+      fterr_warnx(
+        "%s line %d: Only one set per source/destination per match.",
+        lp->fname, lp->lineno);
+      return -1;
+    }
+
+    interface_look->set_flags_lookup[interface] |= sflag;
+
+    if (sflag & FT_TAG_SET_SRC_TAG)
+      interface_look->src_tag_lookup[interface] = tag;
+
+    if (sflag & FT_TAG_OR_SRC_TAG)
+      interface_look->src_tag_lookup[interface] |= tag;
+  
+    if (sflag & FT_TAG_SET_DST_TAG)
+      interface_look->dst_tag_lookup[interface] = tag;
+  
+    if (sflag & FT_TAG_OR_DST_TAG)
+      interface_look->dst_tag_lookup[interface] |= tag;
+
+
   } else if (lp->cur_action->type &
     (FT_TAG_TYPE_MATCH_TCP_PORT|FT_TAG_TYPE_MATCH_UDP_PORT)) {
 
     port = atoi(match);
 
-    if (port_look->set_flags_lookup[port] & sflag) {
-      fterr_warnx("%s line %d: Only one match.", lp->fname, lp->lineno);
+    tmpflag = port_look->set_flags_lookup[port];
+
+    if (((tmpflag & FT_TAG_SET_SRCDST) & tflag) ||
+       ((tmpflag & FT_TAG_OR_SRCDST) & tflag2)) {
+      fterr_warnx(
+        "%s line %d: Only one set per source/destination per match.",
+        lp->fname, lp->lineno);
       return -1;
     }
 
@@ -702,10 +887,10 @@ int parse_action_match(struct line_parser *lp, struct fttag *fttag)
 
     if (sflag & FT_TAG_OR_SRC_TAG)
       port_look->src_tag_lookup[port] |= tag;
-
+  
     if (sflag & FT_TAG_SET_DST_TAG)
       port_look->dst_tag_lookup[port] = tag;
-
+  
     if (sflag & FT_TAG_OR_DST_TAG)
       port_look->dst_tag_lookup[port] |= tag;
 
@@ -713,8 +898,13 @@ int parse_action_match(struct line_parser *lp, struct fttag *fttag)
 
     tos = atoi(match);
 
-    if (tos_look->set_flags_lookup[tos] & sflag) {
-      fterr_warnx("%s line %d: Only one match.", lp->fname, lp->lineno);
+    tmpflag = tos_look->set_flags_lookup[tos];
+
+    if (((tmpflag & FT_TAG_SET_SRCDST) & tflag) ||
+       ((tmpflag & FT_TAG_OR_SRCDST) & tflag2)) {
+      fterr_warnx(
+        "%s line %d: Only one set per source/destination per match.",
+        lp->fname, lp->lineno);
       return -1;
     }
 
@@ -739,8 +929,13 @@ int parse_action_match(struct line_parser *lp, struct fttag *fttag)
       return -1;
     }
 
-    if (any_look->set_flags & sflag) {
-      fterr_warnx("%s line %d: Only one match.", lp->fname, lp->lineno);
+    tmpflag = any_look->set_flags;
+
+    if (((tmpflag & FT_TAG_SET_SRCDST) & tflag) ||
+       ((tmpflag & FT_TAG_OR_SRCDST) & tflag2)) {
+      fterr_warnx(
+        "%s line %d: Only one set per source/destination per match.",
+        lp->fname, lp->lineno);
       return -1;
     }
 
@@ -776,9 +971,13 @@ int parse_action_match(struct line_parser *lp, struct fttag *fttag)
     if (prefix_look && (prefix_look->addr.sin_addr.s_addr == ipp.addr) && 
        (prefix_look->masklen == ipp.len)) {
 
-      /* if the set is invalid fail, else update the record and return */
-      if (prefix_look->set_flags & sflag) {
-        fterr_warnx("%s line %d: Only one match.", lp->fname, lp->lineno);
+      tmpflag = prefix_look->set_flags;
+
+      if (((tmpflag & FT_TAG_SET_SRCDST) & tflag) ||
+         ((tmpflag & FT_TAG_OR_SRCDST) & tflag2)) {
+        fterr_warnx(
+          "%s line %d: Only one set per source/destination per match.",
+          lp->fname, lp->lineno);
         return -1;
       } else {
         new = 0;
@@ -823,13 +1022,13 @@ int parse_action_match(struct line_parser *lp, struct fttag *fttag)
 
     if (sflag & FT_TAG_SET_SRC_TAG)
       prefix_look->src_tag = tag;
-
+        
     if (sflag & FT_TAG_OR_SRC_TAG)
       prefix_look->src_tag |= tag;
-
+      
     if (sflag & FT_TAG_SET_DST_TAG)
       prefix_look->dst_tag = tag;
-
+      
     if (sflag & FT_TAG_OR_DST_TAG)
       prefix_look->dst_tag |= tag;
 
@@ -848,9 +1047,13 @@ int parse_action_match(struct line_parser *lp, struct fttag *fttag)
     /* if it exists, make sure not a duplicate set */
     if (nh_look) {
 
-      /* if the set is invalid fail, else update the record and return */
-      if (nh_look->set_flags & sflag) {
-        fterr_warnx("%s line %d: Only one match.", lp->fname, lp->lineno);
+      tmpflag = nh_look->set_flags;
+
+      if (((tmpflag & FT_TAG_SET_SRCDST) & tflag) ||
+         ((tmpflag & FT_TAG_OR_SRCDST) & tflag2)) {
+        fterr_warnx(
+          "%s line %d: Only one set per source/destination per match.",
+          lp->fname, lp->lineno);
         return -1;
       } else {
         new = 0;
@@ -877,7 +1080,7 @@ int parse_action_match(struct line_parser *lp, struct fttag *fttag)
 
     if (sflag & FT_TAG_SET_SRC_TAG)
       nh_look->src_tag = tag;
-
+    
     if (sflag & FT_TAG_OR_SRC_TAG)
       nh_look->src_tag |= tag;
 
@@ -887,6 +1090,122 @@ int parse_action_match(struct line_parser *lp, struct fttag *fttag)
     if (sflag & FT_TAG_OR_DST_TAG)
       nh_look->dst_tag |= tag;
 
+  } else if (lp->cur_action->type & FT_TAG_TYPE_MATCH_EXPORTER) {
+
+    ipaddr = scan_ip(match);
+
+    hash = (ipaddr>>16) ^ (ipaddr & 0xFFFF);
+    hash = (hash>>8) ^ (hash & 0xFF);
+
+    exporter_look = ftchash_lookup(lp->cur_action->look, &ipaddr, hash);
+
+    new = 1;
+
+    /* if it exists, make sure not a duplicate set */
+    if (exporter_look) {
+
+      tmpflag = exporter_look->set_flags;
+
+      if (((tmpflag & FT_TAG_SET_SRCDST) & tflag) ||
+         ((tmpflag & FT_TAG_OR_SRCDST) & tflag2)) {
+        fterr_warnx(
+          "%s line %d: Only one set per source/destination per match.",
+          lp->fname, lp->lineno);
+        return -1;
+      } else {
+        new = 0;
+      }
+
+    }
+
+    if (new) {
+
+      bzero(&exporter_look2, sizeof exporter_look2);
+
+      exporter_look2.addr = ipaddr;
+
+      if (!(exporter_look = ftchash_update(lp->cur_action->look,
+        &exporter_look2, hash))) {
+        fterr_warnx("ftch_update(): failed");
+        return -1;
+      }
+
+    }
+
+    /* finish filling in fields */
+
+    exporter_look->set_flags |= sflag;
+
+    if (sflag & FT_TAG_SET_SRC_TAG)
+      exporter_look->src_tag = tag;
+
+    if (sflag & FT_TAG_OR_SRC_TAG)
+      exporter_look->src_tag |= tag;
+      
+    if (sflag & FT_TAG_SET_DST_TAG)
+      exporter_look->dst_tag = tag;
+
+    if (sflag & FT_TAG_OR_DST_TAG)
+      exporter_look->dst_tag |= tag;
+
+  } else if (lp->cur_action->type & FT_TAG_TYPE_MATCH_IP) {
+
+    ipaddr = scan_ip(match);
+
+    hash = (ipaddr>>16) ^ (ipaddr & 0xFFFF);
+    hash = (hash>>8) ^ (hash & 0xFF);
+
+    ip_look = ftchash_lookup(lp->cur_action->look, &ipaddr, hash);
+
+    new = 1;
+
+    /* if it exists, make sure not a duplicate set */
+    if (ip_look) {
+
+      tmpflag = ip_look->set_flags;
+
+      if (((tmpflag & FT_TAG_SET_SRCDST) & tflag) ||
+         ((tmpflag & FT_TAG_OR_SRCDST) & tflag2)) {
+        fterr_warnx(
+          "%s line %d: Only one set per source/destination per match.",
+          lp->fname, lp->lineno);
+        return -1;
+      } else {
+        new = 0;
+      }
+
+    }
+
+    if (new) {
+
+      bzero(&ip_look2, sizeof ip_look2);
+
+      ip_look2.addr = ipaddr;
+
+      if (!(ip_look = ftchash_update(lp->cur_action->look,
+        &ip_look2, hash))) {
+        fterr_warnx("ftch_update(): failed");
+        return -1;
+      }
+
+    }
+
+    /* finish filling in fields */
+
+    ip_look->set_flags |= sflag;
+
+    if (sflag & FT_TAG_SET_SRC_TAG)
+      ip_look->src_tag = tag;
+
+    if (sflag & FT_TAG_OR_SRC_TAG)
+      ip_look->src_tag |= tag;
+      
+    if (sflag & FT_TAG_SET_DST_TAG)
+      ip_look->dst_tag = tag;
+       
+    if (sflag & FT_TAG_OR_DST_TAG)
+      ip_look->dst_tag |= tag;
+
   }
 
   return 0;
@@ -1252,6 +1571,10 @@ void fttag_free(struct fttag *fttag)
       free(fta->look);
     else if (fta->type & FT_TAG_TYPE_MATCH_NEXTHOP)
       ftchash_free(fta->look);
+    else if (fta->type & FT_TAG_TYPE_MATCH_EXPORTER)
+      ftchash_free(fta->look);
+    else if (fta->type & FT_TAG_TYPE_MATCH_IP)
+      ftchash_free(fta->look);
     else if (fta->type & FT_TAG_TYPE_MATCH_PREFIX) {
       rhead = fta->look;
       rhead->rnh_walktree(rhead, walk_free, 0);
@@ -1502,6 +1825,114 @@ inline void eval_match_nexthop(struct fttag_action *fta,
 
 } /* eval_match_nexthop */
 
+inline void eval_match_exporter(struct fttag_action *fta,
+  struct fts3rec_v1005 *rec)
+{
+  struct ftchash *ftch;
+  u_int32 hash, ipaddr;
+  struct fttag_exporter_look *exporter_look;
+  u_int16 set_tmp;
+
+  ftch = fta->look;
+  ipaddr = rec->exaddr;
+
+  hash = (ipaddr>>16) ^ (ipaddr & 0xFFFF);
+  hash = (hash>>8) ^ (hash & 0xFF);
+
+  /* lookup next hop */
+  exporter_look = ftchash_lookup(ftch, &ipaddr, hash);
+
+  if (exporter_look) {
+
+    set_tmp = exporter_look->set_flags;
+
+
+    if (set_tmp & FT_TAG_SET_DST_TAG)
+      rec->dst_tag = exporter_look->dst_tag;
+    else if (set_tmp & FT_TAG_OR_DST_TAG)
+      rec->dst_tag |= exporter_look->dst_tag;
+ 
+    if (set_tmp & FT_TAG_SET_SRC_TAG)
+      rec->src_tag = exporter_look->src_tag;
+    else if (set_tmp & FT_TAG_OR_SRC_TAG)
+      rec->src_tag |= exporter_look->src_tag;   
+
+  }
+
+} /* eval_match_exporter */
+
+inline void eval_match_src_ip(struct fttag_action *fta,
+  struct fts3rec_v1005 *rec)
+{
+  struct ftchash *ftch;
+  u_int32 hash, ipaddr;
+  struct fttag_ip_look *ip_look;
+  u_int16 set_tmp;
+
+  ftch = fta->look;
+  ipaddr = rec->srcaddr;
+
+  hash = (ipaddr>>16) ^ (ipaddr & 0xFFFF);
+  hash = (hash>>8) ^ (hash & 0xFF);
+
+  /* lookup next hop */
+  ip_look = ftchash_lookup(ftch, &ipaddr, hash);
+
+  if (ip_look) {
+
+    set_tmp = ip_look->set_flags;
+
+
+    if (set_tmp & FT_TAG_SET_DST_TAG)
+      rec->dst_tag = ip_look->dst_tag;
+    else if (set_tmp & FT_TAG_OR_DST_TAG)
+      rec->dst_tag |= ip_look->dst_tag;
+ 
+    if (set_tmp & FT_TAG_SET_SRC_TAG)
+      rec->src_tag = ip_look->src_tag;
+    else if (set_tmp & FT_TAG_OR_SRC_TAG)
+      rec->src_tag |= ip_look->src_tag;   
+
+  }
+
+} /* eval_match_src_ip */
+
+inline void eval_match_dst_ip(struct fttag_action *fta,
+  struct fts3rec_v1005 *rec)
+{
+  struct ftchash *ftch;
+  u_int32 hash, ipaddr;
+  struct fttag_ip_look *ip_look;
+  u_int16 set_tmp;
+
+  ftch = fta->look;
+  ipaddr = rec->dstaddr;
+
+  hash = (ipaddr>>16) ^ (ipaddr & 0xFFFF);
+  hash = (hash>>8) ^ (hash & 0xFF);
+
+  /* lookup next hop */
+  ip_look = ftchash_lookup(ftch, &ipaddr, hash);
+
+  if (ip_look) {
+
+    set_tmp = ip_look->set_flags;
+
+
+    if (set_tmp & FT_TAG_SET_DST_TAG)
+      rec->dst_tag = ip_look->dst_tag;
+    else if (set_tmp & FT_TAG_OR_DST_TAG)
+      rec->dst_tag |= ip_look->dst_tag;
+ 
+    if (set_tmp & FT_TAG_SET_SRC_TAG)
+      rec->src_tag = ip_look->src_tag;
+    else if (set_tmp & FT_TAG_OR_SRC_TAG)
+      rec->src_tag |= ip_look->src_tag;   
+
+  }
+
+} /* eval_match_dst_ip */
+
 inline void eval_match_prefix(struct fttag_action *fta,
   struct fts3rec_v1005 *rec)
 {
@@ -1634,6 +2065,64 @@ inline void eval_match_udp_port(struct fttag_action *fta,
   eval_match_udp_dst_port(fta, rec);
 } /* eval_match_udp_port */
 
+inline void eval_match_in_interface(struct fttag_action *fta,
+  struct fts3rec_v1005 *rec)
+{
+  struct fttag_interface_look *interface_look;
+  u_int16 set_tmp;
+ 
+  interface_look = fta->look;
+
+  set_tmp = interface_look->set_flags_lookup[rec->input];
+ 
+  if (set_tmp & FT_TAG_SET_DST_TAG)
+    rec->dst_tag = interface_look->dst_tag_lookup[rec->input];
+  else if (set_tmp & FT_TAG_OR_DST_TAG)
+    rec->dst_tag |= interface_look->dst_tag_lookup[rec->input];
+ 
+  if (set_tmp & FT_TAG_SET_SRC_TAG)
+    rec->src_tag = interface_look->src_tag_lookup[rec->input];
+  if (set_tmp & FT_TAG_OR_SRC_TAG)
+    rec->src_tag |= interface_look->src_tag_lookup[rec->input];
+
+} /* eval_match_in_interface */
+
+inline void eval_match_out_interface(struct fttag_action *fta,
+  struct fts3rec_v1005 *rec)
+{
+  struct fttag_interface_look *interface_look;
+  u_int16 set_tmp;
+ 
+  interface_look = fta->look;
+
+  set_tmp = interface_look->set_flags_lookup[rec->output];
+ 
+  if (set_tmp & FT_TAG_SET_DST_TAG)
+    rec->dst_tag = interface_look->dst_tag_lookup[rec->output];
+  else if (set_tmp & FT_TAG_OR_DST_TAG)
+    rec->dst_tag |= interface_look->dst_tag_lookup[rec->output];
+ 
+  if (set_tmp & FT_TAG_SET_SRC_TAG)
+    rec->src_tag = interface_look->src_tag_lookup[rec->output];
+  if (set_tmp & FT_TAG_OR_SRC_TAG)
+    rec->src_tag |= interface_look->src_tag_lookup[rec->output];
+
+} /* eval_match_out_interface */
+
+inline void eval_match_interface(struct fttag_action *fta,
+  struct fts3rec_v1005 *rec)
+{
+  eval_match_in_interface(fta, rec);
+  eval_match_out_interface(fta, rec);
+} /* eval_match_interface */
+
+inline void eval_match_ip(struct fttag_action *fta,
+  struct fts3rec_v1005 *rec)
+{
+  eval_match_src_ip(fta, rec);
+  eval_match_dst_ip(fta, rec);
+} /* eval_match_ip */
+
 inline void eval_match_tos(struct fttag_action *fta,
   struct fts3rec_v1005 *rec)
 {
