@@ -23,11 +23,11 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *      $Id: flow-capture.c,v 1.72 2003/02/24 02:55:15 maf Exp $
+ *      $Id: flow-capture.c,v 1.77 2003/11/29 06:43:08 maf Exp $
  */
 
 #include "ftconfig.h"
-#include "ftlib.h"
+#include <ftlib.h>
 
 #include <sys/time.h>
 #include <sys/types.h>
@@ -108,12 +108,13 @@ struct file {
 };
 
 int debug;
-int sig_pipe_flag, sig_quit_flag, sig_hup_flag, sig_chld_flag;
+int sig_pipe_flag, sig_quit_flag, sig_hup_flag, sig_chld_flag, sig_term_flag;
 int reload_flag;
 void sig_pipe(int);
 void sig_quit(int);
 void sig_hup(int);
 void sig_chld(int);
+void sig_term(int);
 pid_t pid;
 char *pidfile;
 struct ftnet ftnet;
@@ -147,11 +148,12 @@ char **argv;
   struct ftchash_rec_exp ftch_recexp, *ftch_recexpp;
   struct rotate rot;
   struct file cap_file;
-  struct ftipmask ftipmask;
   struct fttag fttag;
   struct ftfil ftfil;
+  struct ftxlate ftxlate;
   struct fttag_def *ftd;
   struct ftfil_def *ftfd;
+  struct ftxlate_def *ftxd;
   struct fts3rec_offsets fo;
   struct client_rec *client_rec, *client_rec2;
   pid_t child_pid;
@@ -160,13 +162,15 @@ char **argv;
   char work_dir[MAXPATHLEN+1], post_rotate_exec[MAXPATHLEN+1];
   int i, n, tmp_len, enable_unlink, offset, detach, nest, one, max_fd;
   unsigned int v1, v2;
-  u_int32 privacy_mask, hash;
+  u_int32 hash;
   char fmt_src_ip[32], fmt_dst_ip[32], fmt_dst_port[32];
   char xl_rec[FT_IO_MAXREC], *out_rec;
   char *tag_fname, *tag_active;
   char *filter_fname, *filter_active;
+  char *xlate_fname, *xlate_active;
   int stat_interval, stat_next, child_status;
   int v_flag;
+  int preserve_umask;
 
   time_startup = time((time_t)0L);
 
@@ -182,11 +186,13 @@ char **argv;
   bzero (&ftv, sizeof ftv);
   bzero (&fttag, sizeof fttag);
   bzero (&ftfil, sizeof ftfil);
+  bzero (&ftxlate, sizeof ftxlate);
   FT_LIST_INIT(&client.list);
   stat_interval = 0;
   stat_next = -1;
   v_flag = 0;
   reload_flag = 1;
+  preserve_umask = 0;
 
   tag_fname = FT_PATH_CFG_TAG;
   tag_active = (char*)0L;
@@ -194,8 +200,12 @@ char **argv;
   filter_fname = FT_PATH_CFG_FILTER;
   filter_active = (char*)0L;
 
+  xlate_fname = FT_PATH_CFG_XLATE;
+  xlate_active = (char*)0L;
+
   ftfd = (struct ftfil_def*)0L;
   ftd = (struct fttag_def*)0L;
+  ftxd = (struct ftxlate_def*)0L;
 
   /* init fterr */
   fterr_setid(argv[0]);
@@ -204,9 +214,6 @@ char **argv;
   /* defaults + default compression */
   ftset_init(&ftset, Z_DEFAULT_COMPRESSION);
 
-  /* by default do not mask any src/dst ip addr bits */
-  privacy_mask = 0xFFFFFFFF;
-
   /* default timeout waiting for an active fd */
   tv.tv_sec = SELECT_TIMEOUT;
 
@@ -235,15 +242,11 @@ char **argv;
   /* pidfile */
   pidfile = CAPTURE_PIDFILE;
 
-  while ((i = getopt(argc, argv, "A:b:c:C:d:De:E:f:F:hm:n:N:p:S:t:T:V:w:z:R:"))
-    != -1)
+  while ((i = getopt(argc, argv,
+    "b:c:C:d:De:E:f:F:hn:N:p:S:t:T:uV:w:x:X:z:R:")) != -1)
   
     switch (i) {
 
-    case 'A': /* AS substitution */
-      ftset.as_sub = atoi(optarg);
-      break;
-  
     case 'b': /* output byte order */
       if (!strcasecmp(optarg, "little"))
         ftset.byte_order = FT_HEADER_LITTLE_ENDIAN;
@@ -296,10 +299,6 @@ char **argv;
       exit (0);
       break;
 
-    case 'm': /* privacy mask */
-      privacy_mask = scan_ip(optarg);
-      break;
-
     case 'n': /* # rotations / day */
       rot.n = atoi(optarg);
       /* no more than 1 rotation per minute */
@@ -342,7 +341,10 @@ char **argv;
       ftv.s_version = FT_IO_SVERSION;
       ftv.d_version = 1005;
       ftv.set = 1;
-    
+      break;
+
+    case 'i': /* preserve inherited umask */
+      preserve_umask = 1;
       break;
       
     case 'V': /* PDU version */
@@ -369,6 +371,14 @@ char **argv;
       strcpy(work_dir, optarg);
       break;
 
+    case 'x': /* xlate file name */
+      xlate_fname = optarg;
+      break;
+  
+    case 'X': /* xlate definition name */
+      xlate_active = optarg;
+      break;
+
     case 'z': /* compress level */
       ftset.z_level = atoi(optarg);
       if ((ftset.z_level < 0) || (ftset.z_level > 9))
@@ -416,11 +426,6 @@ char **argv;
   else
     enable_unlink = 1;
 
-  /* setup for ftrec_mask_ip */
-  if (privacy_mask != 0xFFFFFFFF)
-    ftrec_compute_mask(&ftipmask, privacy_mask, privacy_mask,
-      ftset.byte_order);
-
   /* daemonize */
   if (detach) {
     if ((pid = fork()) == -1) {
@@ -431,8 +436,11 @@ char **argv;
       exit (0); /* parent */
     }
 
-    umask(0022);
+    if (!preserve_umask)
+      umask(0022);
+
     setsid();
+
     for (n = 0; n < 16; ++n)
       close (n);
 
@@ -457,6 +465,9 @@ char **argv;
   if (mysignal(SIGQUIT, sig_quit) == SIG_ERR)
     fterr_err(1, "signal(SIGQUIT)");
 
+  if (mysignal(SIGTERM, sig_term) == SIG_ERR)
+    fterr_err(1, "signal(SIGTERM)");
+
   if (mysignal(SIGCHLD, sig_chld) == SIG_ERR)
     fterr_err(1, "signal(SIGCHLD)");
 
@@ -631,7 +642,8 @@ mcast_done:
       /* too many clients? */
       if (client.active >= client.max) {
         /* bye */
-        i = accept(client.fd, (struct sockaddr*)&tmp_addr, &tmp_len);
+        i = accept(client.fd, (struct sockaddr*)&tmp_addr,
+          (socklen_t*)&tmp_len);
         close(i);
         fterr_warnx("Maximum clients exceeded, rejecting connection from %s",
         inet_ntoa(tmp_addr.sin_addr));
@@ -643,7 +655,8 @@ mcast_done:
         fterr_warn("malloc()");
         /* bye */
         tmp_len = sizeof (struct sockaddr_in);
-        i = accept(client.fd, (struct sockaddr*)&client_rec->addr, &tmp_len);
+        i = accept(client.fd, (struct sockaddr*)&client_rec->addr,
+          (socklen_t*)&tmp_len);
         close(i);
         goto skip_client;
       }
@@ -656,7 +669,7 @@ mcast_done:
       tmp_len = sizeof (struct sockaddr_in);
       /* accept() the connection */
       if ((client_rec->fd = accept(client.fd,
-        (struct sockaddr*)&client_rec->addr, &tmp_len)) < 0) {
+        (struct sockaddr*)&client_rec->addr, (socklen_t*)&tmp_len)) < 0) {
         fterr_warn("accept()");
         FT_LIST_REMOVE(client_rec, chain);
         free(client_rec);
@@ -954,7 +967,6 @@ restart_recvmsg:
 
       /* decode the pdu */
       ftpdu.ftd.byte_order = ftset.byte_order;
-      ftpdu.ftd.as_sub = ftset.as_sub;
       ftpdu.ftd.exporter_ip = ftch_recexp.src_ip;
       n = fts3rec_pdu_decode(&ftpdu);
 
@@ -1065,6 +1077,26 @@ skip_pdu_decode:
     
       } /* filter_active */
 
+      /* load translations */
+      if (xlate_active) {
+  
+        /* not first time through, then free previous translations */
+        if (ftxd) {
+          ftxlate_free(&ftxlate);
+          fterr_info("Reloading translations.");
+        }
+        
+        if (ftxlate_load(&ftxlate, xlate_fname))
+          fterr_errx(1, "ftxlate_load(%s): failed", xlate_fname);
+            
+        if (!(ftxd = ftxlate_def_find(&ftxlate, xlate_active)))
+          fterr_errx(1, "ftlate_def_find(%s): failed", xlate_active);
+
+        if (ftxlate_def_test_xfields(ftxd, ftrec_xfield(&ftv)))
+          fterr_errx(1, "Xlate references a field not in flow.");
+    
+      } /* xlate_active */
+
       reload_flag = 0;
 
     } /* reload_flag */
@@ -1097,13 +1129,14 @@ skip_pdu_decode:
           continue; 
         }
 
+      /* xlate? */
+      if (ftxd)
+        if (ftxlate_def_eval(ftxd, out_rec, &fo) != 0)
+          fterr_errx(1, "ftxlate_def_eval(): failed.");
+
       /* update # of flows stored in capture file */
       cap_file.hdr_nflows ++;
 
-      /* simple data privacy */
-      if (privacy_mask != 0xFFFFFFFF)
-        ftrec_mask_ip(out_rec, &ftv, &ftipmask);
- 
       if ((n = ftio_write(&ftio, out_rec)) < 0)
         fterr_errx(1, "ftio_write(): failed");
 
@@ -1138,7 +1171,7 @@ skip_pdu_decode:
     /*
      * time for a new file ?
      */
-    if ((now > rot.next) || sig_quit_flag || sig_hup_flag) {
+    if ((now > rot.next) || sig_term_flag || sig_quit_flag || sig_hup_flag) {
 
       if (sig_hup_flag)
         fterr_info("SIGHUP");
@@ -1206,14 +1239,14 @@ skip_pdu_decode:
         cap_file.fd = -1;
 
         /* had enough */
-        if (sig_quit_flag)
+        if (sig_quit_flag || sig_term_flag)
           goto main_exit;
       } /* file open */
 
     } /* time for new file */
 
     /* also need to check sig_quit if no file has been processed yet */
-    if (sig_quit_flag)
+    if (sig_quit_flag || sig_term_flag)
       goto main_exit;
 
   /*
@@ -1267,6 +1300,9 @@ main_exit:
   if (sig_quit_flag)
     fterr_info("SIGQUIT");
 
+  if (sig_term_flag)
+    fterr_info("SIGTERM");
+
   /* free storage allocated to file entries */
   if (fte.expiring)
     ftfile_free(&fte);
@@ -1291,6 +1327,11 @@ void sig_quit(int signo)
   sig_quit_flag = 1;
 }
 
+void sig_term(int signo)
+{  
+  sig_term_flag = 1;
+}
+
 void sig_chld(int signo)
 {  
   sig_chld_flag = 1;
@@ -1362,12 +1403,13 @@ void fterr_exit_handler(int code)
 
 void usage(void) {
 
-  fprintf(stderr, "Usage: flow-capture [-h] [-A AS0_substitution] [-b big|little]\n");
+  fprintf(stderr, "Usage: flow-capture [-hu] [-b big|little]\n");
   fprintf(stderr, "       [-C comment] [-c flow_clients] [-d debug_level] [-D daemonize]\n");
-  fprintf(stderr, "       [-e expire_count] [-E expire_size[bKMG]] [-m privacy_mask]\n");
-  fprintf(stderr, "       [-n rotations] [-N nesting_level] [-p pidfile ] [-R rotate_program]\n");
+  fprintf(stderr, "       [-e expire_count] [-E expire_size[bKMG]] [-n rotations]\n");
+  fprintf(stderr, "       [-N nesting_level] [-p pidfile ] [-R rotate_program]\n");
   fprintf(stderr, "       [-S stat_interval] [-t tag_fname] [-T tag_active] [-V pdu_version]\n");
-  fprintf(stderr, "       [-z z_level] -w workdir localip/remoteip/port\n");
+  fprintf(stderr, "       [-z z_level] [-x xlate_fname] [-X xlate_active]\n");
+  fprintf(stderr, "       -w workdir localip/remoteip/port\n");
   fprintf(stderr, "Signals:\n");
   fprintf(stderr, "   SIGHUP  - close and rotate current file\n");
   fprintf(stderr, "   SIGQUIT - close current file and exit\n");
