@@ -23,11 +23,11 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *      $Id: flow-dscan.c,v 1.24 2003/02/13 02:38:59 maf Exp $
+ *      $Id: flow-dscan.c,v 1.28 2003/11/25 10:14:27 maf Exp $
  */
 
 #include "ftconfig.h"
-#include "ftlib.h"
+#include <ftlib.h>
 
 #include <sys/time.h>
 #include <sys/types.h>
@@ -105,7 +105,7 @@ int main(int argc, char **argv)
   struct dscan_dst *ddp, *ddp2;
   struct dscan_sup *dsp_src, *dsp_dst;
   u_long hash;
-  char fmt_buf1[64], fmt_buf2[64], fmt_buf3[64], fmt_buf4[54];
+  char fmt_buf1[64], fmt_buf2[64], fmt_buf3[64], fmt_buf4[64];
   int do_dump, do_load;
   int filter_www, filter_mcast, filter_input, filter_output;
   char in_tbl[65536], out_tbl[65536];
@@ -176,7 +176,7 @@ int main(int argc, char **argv)
       break;
 
     case 'w': /* filter www inbound (replies) */
-      filter_www = 1;
+      filter_www |= 1;
       break;
 
     case 'b': /* no detach */
@@ -212,7 +212,7 @@ int main(int argc, char **argv)
       break;
 
     case 'W': /* filter www outbound */
-      filter_www = 2;
+      filter_www |= 2;
       break;
 
     case 'h': /* help */
@@ -352,11 +352,11 @@ int main(int argc, char **argv)
     /* calculate hash based on src ip */
     hash = DSCAN_HASHFUNC(rec->srcaddr);
 
-    if ((filter_www == 1) && (rec->srcport == 80) && (rec->prot == 6) &&
+    if ((filter_www & 1) && (rec->srcport == 80) && (rec->prot == 6) &&
       (rec->dstport > 1023))
       goto skip2;
 
-    if ((filter_www == 2) && (rec->dstport == 80) && (rec->prot == 6) &&
+    if ((filter_www & 2) && (rec->dstport == 80) && (rec->prot == 6) &&
       (rec->srcport > 1023))
       goto skip2;
 
@@ -657,6 +657,7 @@ int load_state(struct dscan_state *ds)
         bzero(drp, sizeof (struct dscan_rec));
         drp->ip_src = ip;
         drp->depth = depth;
+        drp->flags = flags;
         FT_SLIST_INSERT_HEAD(&ds->hash_scan[hash], drp, chain);
 
         /* init destination list */
