@@ -23,11 +23,11 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *      $Id: flow-export.c,v 1.23 2003/02/13 02:38:59 maf Exp $
+ *      $Id: flow-export.c,v 1.25 2003/12/04 05:55:31 maf Exp $
  */
 
 #include "ftconfig.h"
-#include "ftlib.h"
+#include <ftlib.h>
 
 #include <sys/time.h>
 #include <sys/types.h>
@@ -60,6 +60,19 @@
 
 #endif /* MYSQL */
 
+#ifdef HAVE_PGSQL
+
+#include <libpq-fe.h>
+
+#define DB_DEFAULT_DBHOST "localhost"
+#define DB_DEFAULT_DBNAME "netflow"
+#define DB_DEFAULT_DBPORT "5432"
+#define DB_DEFAULT_DBTABLE "raw"
+#define DB_DEFAULT_DBUSER "netflow"
+#define DB_DEFAULT_DBPWD "netflow"
+
+#endif /* PGSQL*/
+
 #if HAVE_LL_STRTOUL
   #define strtoull strtoul
 #endif /* HAVE_LL_STRTOULL */
@@ -88,6 +101,7 @@ int format1(struct ftio *ftio, struct options *opt);
 int format2(struct ftio *ftio, struct options *opt);
 int format3(struct ftio *ftio, struct options *opt);
 int format4(struct ftio *ftio, struct options *opt);
+int format5(struct ftio *ftio, struct options *opt);
 
 int ftxfield_tocflow(u_int64 xfields, u_int32 *cfmask);
 
@@ -97,9 +111,9 @@ int fmt_xfields_type(char *buf, u_int64 xfield);
 
 void usage(void);
 
-#define NFORMATS 5 /* nformats - 1 */
+#define NFORMATS 6 /* nformats - 1 */
 struct jump format[] = {{format0}, {format1}, {format2}, {format3},
-                        {format4}};
+                        {format4}, {format5}};
 
 int main(int argc, char **argv)
 {
@@ -743,7 +757,10 @@ int format3(struct ftio *ftio, struct options *opt)
       strcat (query, ") VALUES (");
       strcat (query, values);
       strcat (query, ")");
-printf("field=%s\n val=%s\n query=%s\n", fields, values, query);
+
+      if (debug)
+        fprintf(stderr, "field=%s\n val=%s\n query=%s\n", fields, values,
+          query);
 
       if (mysql_real_query(&mysql, query, strlen(query)) != 0) 
         fterr_warnx("mysql_real_query(): %s", mysql_error(&mysql));
@@ -835,6 +852,112 @@ retry:
 
 } /* format4 */
 
+/*
+ * function: format5
+ *
+ * export flows into PostgreSQL Database
+ */
+int format5(struct ftio *ftio, struct options *opt)
+{
+#ifdef HAVE_PGSQL
+  struct fts3rec_offsets fo;
+  struct ftver ftv;
+  char fields[1024], values[1024], query[3*1024];
+  char *rec;
+  char *db_host, *db_name, *db_table, *db_user, *db_pwd, *db_tmp, *tmp;
+  char *db_port;
+  int len;
+
+  PGconn     *conn;
+  PGresult   *res;
+
+  db_host = DB_DEFAULT_DBHOST;
+  db_name = DB_DEFAULT_DBNAME;
+  db_port = DB_DEFAULT_DBPORT;
+  db_user = DB_DEFAULT_DBUSER;
+  db_table = DB_DEFAULT_DBTABLE;
+  db_pwd = DB_DEFAULT_DBPWD;
+
+  /* parse URI string */
+
+  if (strlen(opt->dbaseURI)) {
+
+    tmp = opt->dbaseURI;
+
+    db_user = strsep(&tmp, ":");
+    db_pwd = strsep(&tmp, ":");
+    db_host = strsep(&tmp, ":");
+    db_port = strsep(&tmp, ":");
+    db_name = strsep(&tmp, ":");
+    db_table = strsep(&tmp, ":");
+
+    if (!db_user || !db_pwd || !db_host || !db_tmp || !db_name || !db_table) {
+      fterr_warnx("Missing field in dbaseURI, expecting user:pwd:host:port:name:table.");
+      return -1;
+    }
+
+  } /* dbaseURI */
+
+  ftio_get_ver(ftio, &ftv);
+
+  fts3rec_compute_offsets(&fo, &ftv);
+
+  /* remove invalid fields */
+  opt->ft_mask &= ftrec_xfield(&ftv);
+
+  /* generate the field names once */
+  fmt_xfields_type(fields, opt->ft_mask);
+
+  /* open PostgreSQL database */
+  conn = PQsetdbLogin(db_host, db_port, (char *) NULL, (char *) NULL, db_name, db_user, db_pwd);
+
+  if (PQstatus(conn) == CONNECTION_BAD) 
+    fterr_errx(1,"PQsetdbLogin(): %s\n", PQerrorMessage(conn));
+
+  /* foreach flow */
+  while ((rec = ftio_read(ftio))) {
+
+    len = fmt_xfields_val(values, rec, &fo, opt->ft_mask, 1);
+
+    /* form SQL query and execute it */
+    if (len) {
+      strcpy (query, "INSERT INTO ");
+      strcat (query, db_table);
+      strcat (query, "(");
+      strcat (query, fields);
+      strcat (query, ") VALUES (");
+      strcat (query, values);
+      strcat (query, ")");
+
+      if (debug)
+        fprintf(stderr, "field=%s\n val=%s\n query=%s\n", fields, values,
+          query);
+
+      res = PQexec(conn, query);
+      if (!res || PQresultStatus(res) != PGRES_COMMAND_OK) {
+        PQclear(res);
+        fterr_errx(1,"PQexec(): %s\n", PQerrorMessage(conn));
+      }
+
+    }
+
+    ++opt->records;
+
+  } /* while */
+
+  /* close database */
+  PQfinish(conn);
+
+#else /* PGSQL */
+
+  fterr_warnx("Format not supported");
+
+#endif /* PGSQL */
+
+  return 0;
+ 
+} /* format5 */ 
+
 int fmt_xfields_type(char *buf, u_int64 xfield)
 {
   int comma;
@@ -1296,13 +1419,16 @@ int fmt_xfields_val(char *fmt_buf, char *rec, struct fts3rec_offsets *fo,
     comma = 1;
   }
 
+  /* if ended on a quote string may not be null terminated */
+  fmt_buf[len++] = 0;
+
   return strlen(fmt_buf);
 
 } /* fmt_xfields_val */
 
 void usage(void) {
 
-  fprintf(stderr, "Usage: flow-export [-h] [-d debug_level] [-f format] [-m mask_fields] -u [mySQL URI]\n");
+  fprintf(stderr, "Usage: flow-export [-h] [-d debug_level] [-f format] [-m mask_fields] -u [database URI]\n");
   fprintf(stderr, "\n%s version %s: built by %s\n", PACKAGE, VERSION, FT_PROG_BUILD);
 
 
