@@ -23,11 +23,11 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *      $Id: flow-fanout.c,v 1.36 2003/02/24 02:55:15 maf Exp $
+ *      $Id: flow-fanout.c,v 1.41 2003/11/30 23:38:37 maf Exp $
  */
 
 #include "ftconfig.h"
-#include "ftlib.h"
+#include <ftlib.h>
 
 #include <sys/time.h>
 #include <sys/types.h>
@@ -78,10 +78,11 @@ struct peer {
 
 void fterr_exit_handler(int code);
 
-int sig_quit_flag, sig_hup_flag;
-void sig_quit(int), sig_hup(int);
+int sig_quit_flag, sig_hup_flag, sig_term_flag, reload_flag;
+void sig_quit(int), sig_hup(int), sig_term(int);
 pid_t pid;
 u_int16 listen_port;
+
 void pdu_xmit(int npeers, int tx_delay, int src_ip_spoof, int hdr_len,
   u_int32 *send_nobufs, struct ip *ip_hdr, struct udphdr *udp_hdr,
   struct ftencode *fte, struct peer *peers, struct ftnet *ftnet);
@@ -141,6 +142,7 @@ int main(int argc, char **argv)
   src_ip_spoof = 0; /* no */
   hdr_len = 0;
   send_nobufs = 0;
+  reload_flag = 1; /* yes */
 
   filter_fname = FT_PATH_CFG_FILTER;
   filter_active = (char*)0L;
@@ -242,7 +244,6 @@ int main(int argc, char **argv)
   ftencode_init(&fte, (src_ip_spoof) ? FT_ENC_FLAGS_IPHDR : 0);
 
   /* if src ip spoofing, initialize the IP/UDP header */
-
   if (src_ip_spoof) {
 
     ip_hdr = (struct ip*)&fte.buf;
@@ -273,6 +274,9 @@ int main(int argc, char **argv)
   if (mysignal(SIGQUIT, sig_quit) == SIG_ERR)
     fterr_err(1, "signal(SIGQUIT)");
 
+  if (mysignal(SIGTERM, sig_term) == SIG_ERR)
+    fterr_err(1, "signal(SIGTERM)");
+
   if (mysignal(SIGHUP, sig_hup) == SIG_ERR)
     fterr_err(1, "signal(SIGHUP)");
 
@@ -694,6 +698,33 @@ restart_recvmsg:
 
       } /* version processing */
 
+      /* load filters? */
+      if (reload_flag && ftv.set) {
+
+        if (filter_active) {
+
+          /* not first time through, then free previous filters */
+          if (ftfd) {
+            ftfil_free(&ftfil);
+            fterr_info("Loading filters.");
+          }
+
+          if (ftfil_load(&ftfil, filter_fname))
+            fterr_errx(1, "ftfil_load(%s): failed", filter_fname);
+
+          if (!(ftfd = ftfil_def_find(&ftfil, filter_active)))
+            fterr_errx(1, "ftfil_def_find(%s): failed", filter_active);
+      
+          if (ftfil_def_test_xfields(ftfd, ftrec_xfield(&ftv)))
+            fterr_errx(1, "Filter references a field not in flow.");
+        
+        } /* filter_active */
+
+        reload_flag = 0;
+
+      }
+
+
       /* compute 8 bit hash */
       hash = (ftch_recexp.src_ip & 0xFF);
       hash ^= (ftch_recexp.src_ip>>24);
@@ -817,6 +848,11 @@ skip1:
       break;
     }
 
+    if (sig_term_flag) {
+      fterr_info("SIGTERM");
+      break;
+    }
+
     if (sig_hup_flag) {
       fterr_info("SIGHUP");
       sig_hup_flag = 0;
@@ -839,9 +875,15 @@ void sig_quit(int sig)
   sig_quit_flag = 1;
 }
 
+void sig_term(int sig)
+{
+  sig_term_flag = 1;
+}
+
 void sig_hup(int sig)
 {
   sig_hup_flag = 1;
+  reload_flag = 1;
 }
 
 void fterr_exit_handler(int code)
@@ -873,7 +915,7 @@ void pdu_xmit(int npeers, int tx_delay, int src_ip_spoof, int hdr_len,
 /* see Stevens Unix Network Programming Volume 1 2nd edition page 657 */
 /* conditional from <simon@limmat.switch.ch> rawsend.c */
 #if defined (__linux__) || (defined (__OpenBSD__) && (OpenBSD > 199702))
-      ip_hdr->ip_len = htons(FT_ENC_IPHDR_LEN+fte.buf_size);
+      ip_hdr->ip_len = htons(FT_ENC_IPHDR_LEN+fte->buf_size);
 #else
       ip_hdr->ip_len = FT_ENC_IPHDR_LEN+fte->buf_size;
 #endif
