@@ -23,11 +23,11 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *      $Id: flow-receive.c,v 1.50 2003/02/24 02:55:15 maf Exp $
+ *      $Id: flow-receive.c,v 1.52 2003/11/25 05:37:04 maf Exp $
  */
 
 #include "ftconfig.h"
-#include "ftlib.h"
+#include <ftlib.h>
 
 #include <sys/time.h>
 #include <sys/types.h>
@@ -80,7 +80,6 @@ int main(int argc, char **argv)
   struct ftnet ftnet;
   struct ftpeeri ftpi;
   struct ftver ftv;
-  struct ftipmask ftipmask;
   struct ftchash *ftch;
   struct ftchash_rec_exp ftch_recexp, *ftch_recexpp;
   struct fttag fttag;
@@ -95,7 +94,7 @@ int main(int argc, char **argv)
   char *out_fname;
   u_int32 filtered_flows, nflows, time_start, time_end;
   u_int32 flows_corrupt, flows_lost, flows_reset;
-  u_int32 privacy_mask, hash;
+  u_int32 hash;
   char fmt_src_ip[32], fmt_dst_ip[32], fmt_dst_port[32];
   char xl_rec[FT_IO_MAXREC], *out_rec;
   char *tag_fname, *tag_active;
@@ -104,7 +103,6 @@ int main(int argc, char **argv)
 
   time_startup = time((time_t)0L);
 
-  privacy_mask = 0xFFFFFFFF;
   out_fname = (char*)0L;
   filtered_flows = nflows = 0;
   out_fd_plain = 0;
@@ -142,14 +140,10 @@ int main(int argc, char **argv)
   ftnet.loc_addr.sin_addr.s_addr = htonl(INADDR_ANY);
   ftnet.loc_addr.sin_port = htons(FT_PORT);
 
-  while ((i = getopt(argc, argv, "A:b:C:d:f:F:h?m:o:S:t:T:V:z:")) != -1)
+  while ((i = getopt(argc, argv, "b:C:d:f:F:h?o:S:t:T:V:z:")) != -1)
 
     switch (i) {
 
-    case 'A': /* AS substitution */
-      ftset.as_sub = atoi(optarg);
-      break;
-
     case 'b': /* output byte order */
       if (!strcasecmp(optarg, "little"))
         ftset.byte_order = FT_HEADER_LITTLE_ENDIAN;
@@ -181,10 +175,6 @@ int main(int argc, char **argv)
       exit (0);
       break;
 
-    case 'm': /* privacy mask */ 
-      privacy_mask = scan_ip(optarg);
-      break;
-
     case 'o': /* output filename */
       out_fname = optarg;
       break;
@@ -263,11 +253,6 @@ int main(int argc, char **argv)
 
   } /* filter_active */
 
-  /* setup for ftrec_mask_ip */
-  if (privacy_mask != 0xFFFFFFFF)
-    ftrec_compute_mask(&ftipmask, privacy_mask, privacy_mask,
-      ftset.byte_order);
-
   /* number of unprocessed options */
   n = argc - optind;
 
@@ -706,7 +691,6 @@ restart_recvmsg:
 
       /* decode */
       ftpdu.ftd.byte_order = ftset.byte_order;
-      ftpdu.ftd.as_sub = ftset.as_sub;
       ftpdu.ftd.exporter_ip = ftch_recexp.src_ip;  
       n = fts3rec_pdu_decode(&ftpdu);
 
@@ -743,10 +727,6 @@ restart_recvmsg:
 
         ++nflows;
 
-        /* simple data privacy */ 
-        if (privacy_mask != 0xFFFFFFFF)
-          ftrec_mask_ip(out_rec, &ftv, &ftipmask);
-
         if (ftio_write(&ftio, out_rec) < 0)
           fterr_errx(1, "ftio_write(): failed");
 
@@ -796,8 +776,9 @@ void sig_quit(int sig)
 
 void usage(void) {
 
-  fprintf(stderr, "Usage: flow-receive [-h] [-A AS0_substitution] [-b big|little] [-C comment]\n");
-  fprintf(stderr, "       [-d debug_level] [-m privacy_mask] [-o output_file] [-S stat_interval]\n");
+  fprintf(stderr, "Usage: flow-receive [-h] [-b big|little] [-C comment]\n");
+  fprintf(stderr, "       [-d debug_level] [-o output_file] [-S stat_interval]\n");
+  fprintf(stderr, "       [-f filter_name] [-F filter_definition]\n");
   fprintf(stderr, "       [-t tag_fname] [-T tag_active] [-V pdu_version] [-z z_level]\n");
   fprintf(stderr, "       localip/remoteip/port\n");
   fprintf(stderr, "\n%s version %s: built by %s\n", PACKAGE, VERSION, FT_PROG_BUILD);
