@@ -23,11 +23,11 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *      $Id: flow-send.c,v 1.23 2003/02/13 02:39:00 maf Exp $
+ *      $Id: flow-send.c,v 1.26 2003/04/04 02:24:40 maf Exp $
  */
 
 #include "ftconfig.h"
-#include "ftlib.h"
+#include <ftlib.h>
 
 #include <sys/time.h>
 #include <sys/types.h>
@@ -56,9 +56,15 @@ int debug;
 
 void usage(void);
 
+void pdu_xmit(int tx_delay, int src_ip_spoof, int hdr_len,
+  struct ip *ip_hdr, struct udphdr *udp_hdr,
+  int sock, struct ftencode *fte, struct ftpeeri *ftpi);
+
 int main(int argc, char **argv)
 {
   struct sockaddr_in loc_addr, rem_addr;
+  struct ip *ip_hdr;
+  struct udphdr *udp_hdr;
   struct ftio ftio;
   struct ftprof ftp;
   struct ftver ftv, ftv2;
@@ -68,8 +74,9 @@ int main(int argc, char **argv)
   void (*xlate)(void *in_rec, void *out_rec);
   char xl_rec[FT_IO_MAXREC], *out_rec;
   u_int32 privacy_mask;
-  unsigned int v1, v2;
+  unsigned int v1, v2, one;
   int i, n, ret, tx_delay, udp_sock;
+  int src_ip_spoof, hdr_len;
   void *rec;
 
   /* init fterr */
@@ -80,11 +87,12 @@ int main(int argc, char **argv)
   bzero(&rem_addr, sizeof (struct sockaddr_in));
   bzero(&ftv, sizeof ftv);
   privacy_mask = 0xFFFFFFFF;
+  src_ip_spoof = 0; /* no */
 
   /* profile */
   ftprof_start (&ftp);
 
-  while ((i = getopt(argc, argv, "d:h?m:V:x:")) != -1)
+  while ((i = getopt(argc, argv, "d:h?m:sV:x:")) != -1)
     switch (i) {
 
     case 'd': /* debug */
@@ -101,6 +109,10 @@ int main(int argc, char **argv)
       privacy_mask = scan_ip(optarg);
       break;
 
+    case 's': /* source ip preserve */
+      src_ip_spoof = 1;
+      break;
+
     case 'V': /* PDU version */
       n = sscanf(optarg, "%u.%u", &v1, &v2);
       if (n == 1) {
@@ -137,6 +149,10 @@ int main(int argc, char **argv)
   /* get localip/rem_ip/port/ttl */
   ftpi = scan_peeri(argv[optind]);
 
+  /* default ttl to 255 if this is unicast */
+  if (!(IN_CLASSD(ftpi.rem_ip)) && (!ftpi.ttl))
+    ftpi.ttl = 255;
+
   rem_addr.sin_addr.s_addr = htonl(ftpi.rem_ip);
   rem_addr.sin_family = AF_INET;
  
@@ -148,9 +164,31 @@ int main(int argc, char **argv)
   loc_addr.sin_addr.s_addr = htonl(ftpi.loc_ip);
   loc_addr.sin_family = AF_INET;
 
-  /* create a socket for transmitting the flow pdu's */
-  if ((udp_sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
-    fterr_err(1, "socket()");
+    /* if preserving source IP address then these are raw sockets */
+    if (src_ip_spoof) {
+    
+      if ((udp_sock = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) < 0)
+        fterr_err(1, "socket()");
+  
+/* see Stevens Unix Network Programming Volume 1 2nd edition page 657 */
+#ifdef IP_HDRINCL
+  
+      one = 1;
+  
+      if (setsockopt(udp_sock, IPPROTO_IP, IP_HDRINCL,
+        &one, sizeof(one)) < 0) {
+        fterr_err(1, "setsockopt(IP_HDRINCL)");
+      }
+
+#endif /* IP_HDRINCL */
+
+  } else {
+
+    /* normal UDP */
+    if ((udp_sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
+      fterr_err(1, "socket()");
+
+  }
 
   if (bigsockbuf(udp_sock, SO_SNDBUF, FT_SO_SND_BUFSIZE) < 0)
     fterr_err(1, "bigsockbuf()");
@@ -158,7 +196,8 @@ int main(int argc, char **argv)
 #ifdef IP_ADD_MEMBERSHIP
 
   /* multicast destination? */
-  if (IN_CLASSD(ntohl(rem_addr.sin_addr.s_addr))) {
+  if ((!src_ip_spoof) &&
+     (IN_CLASSD(ntohl(rem_addr.sin_addr.s_addr)))) {
  
     u_char ttl = ftpi.ttl;
 
@@ -172,9 +211,12 @@ int main(int argc, char **argv)
 
 #endif /* IP_ADD_MEMBERSHIP */
 
+  if (!src_ip_spoof) {
 
-  if (bind(udp_sock, (struct sockaddr*)&loc_addr, sizeof loc_addr) < 0)
-    fterr_err(1, "bind()");
+    if (bind(udp_sock, (struct sockaddr*)&loc_addr, sizeof loc_addr) < 0)
+      fterr_err(1, "bind()");
+
+  }
     
   if (connect(udp_sock, (struct sockaddr*)&rem_addr, sizeof rem_addr) < 0)
     fterr_err(1, "connect()");
@@ -184,7 +226,20 @@ int main(int argc, char **argv)
     fterr_errx(1, "ftio_init(): failed");
 
   /* initialize encode struct */
-  ftencode_init(&fte, 0);
+  ftencode_init(&fte, (src_ip_spoof) ? FT_ENC_FLAGS_IPHDR : 0);
+
+  /* if src ip spoofing, initialize the IP/UDP header */
+  if (src_ip_spoof) {
+      
+    ip_hdr = (struct ip*)&fte.buf;
+    udp_hdr = (struct udphdr*)((char*)&fte.buf + sizeof (*ip_hdr));
+  
+    ip_hdr->ip_hl = 5;
+    ip_hdr->ip_v = 4;
+    ip_hdr->ip_p = 17; /* UDP */
+    hdr_len = FT_ENC_IPHDR_LEN;
+
+  }
 
   /* copy version from io stream */
   ftio_get_ver(&ftio, &ftv2);
@@ -240,21 +295,8 @@ retry:
 
     if (ret <= 0) {
 
-      /* convert pdu to network byte order */
-#if BYTE_ORDER == LITTLE_ENDIAN
-      ftpdu_swap(fte.buf_enc, BYTE_ORDER);
-#endif /* BYTE_ORDER == LITTLE_ENDIAN */
-
-      if (send(udp_sock, (void*)&fte.buf, fte.buf_size, 0) < 0)
-        if (errno != ECONNREFUSED)
-          fterr_err(1, "send()");
-
-      /* try not to overload send/receive buffers */
-      if (tx_delay)
-        usleep((unsigned)tx_delay);
-
-      /* reset encode buffer */
-      ftencode_reset(&fte);
+      pdu_xmit(tx_delay, src_ip_spoof, hdr_len, ip_hdr, udp_hdr, udp_sock,
+        &fte, &ftpi);
 
       /* if ret < 0 then the current record was not encoded */
       if (ret < 0)
@@ -266,16 +308,8 @@ retry:
   /* any left over? */
   if (fte.buf_size) {
 
-    /* convert pdu to network byte order */
-    ftpdu_swap(fte.buf_enc, BYTE_ORDER);
-
-    if (send(udp_sock, (void*)&fte.buf, fte.buf_size, 0) < 0)
-      if (errno != ECONNREFUSED)
-        fterr_err(1, "send()");
-
-    /* try not to overload send/receive buffers */
-    if (tx_delay)
-      usleep((unsigned)tx_delay);
+    pdu_xmit(tx_delay, src_ip_spoof, hdr_len, ip_hdr, udp_hdr, udp_sock,
+      &fte, &ftpi);
 
   } /* fte.buf_size */
   
@@ -288,6 +322,72 @@ retry:
 
 } /* main */
 
+
+void pdu_xmit(int tx_delay, int src_ip_spoof, int hdr_len,
+  struct ip *ip_hdr, struct udphdr *udp_hdr,
+  int sock, struct ftencode *fte, struct ftpeeri *ftpi)
+{
+  int sum;
+
+  /* convert pdu to network byte order */
+#if BYTE_ORDER == LITTLE_ENDIAN
+  ftpdu_swap(fte->buf_enc, BYTE_ORDER);
+#endif /* BYTE_ORDER == LITTLE_ENDIAN */
+
+  /* do this once */
+  ftencode_sum_data(fte);
+
+  if (src_ip_spoof) {
+
+/* see Stevens Unix Network Programming Volume 1 2nd edition page 657 */
+/* conditional from <simon@limmat.switch.ch> rawsend.c */
+#if defined (__linux__) || (defined (__OpenBSD__) && (OpenBSD > 199702))
+    ip_hdr->ip_len = htons(FT_ENC_IPHDR_LEN+fte->buf_size);
+#else
+    ip_hdr->ip_len = FT_ENC_IPHDR_LEN+fte->buf_size;
+#endif
+    ip_hdr->ip_ttl = ftpi->ttl;
+    ip_hdr->ip_src.s_addr = htonl(ftpi->loc_ip);
+    ip_hdr->ip_dst.s_addr = htonl(ftpi->rem_ip);
+
+    udp_hdr->uh_sport = htons(7999);
+    udp_hdr->uh_dport = htons(ftpi->dst_port);
+    udp_hdr->uh_ulen = htons(fte->buf_size+8);
+    udp_hdr->uh_sum = 0;
+
+    sum = fte->d_sum;
+    sum += udp_cksum(ip_hdr, udp_hdr, fte->buf_size+8);
+
+    sum = (sum >> 16) + (sum & 0xffff);
+    sum += (sum >> 16);
+    udp_hdr->uh_sum = ~sum;
+
+  }
+ 
+again:
+ 
+  if (send(sock, (char*)&fte->buf, fte->buf_size+hdr_len, 0) < 0) {
+
+    /* always complete a send, drop flows in the kernel on receive if
+       overloaded */
+    if (errno == ENOBUFS) {
+      usleep(1);
+      goto again;
+    }
+
+    if (errno != ECONNREFUSED)
+      fterr_warn("send()");
+
+  }
+
+  if (tx_delay)
+    usleep((unsigned)tx_delay);
+
+  /* reset encode buffer */
+  ftencode_reset(fte);
+
+} /* pdu_xmit */
+
 void usage(void) {
 
   fprintf(stderr, "Usage: flow-send [-h] [-d debug_level] [-x xmit_delay] [-V pdu_version]\n");
