@@ -23,11 +23,11 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *      $Id: flow-xlate.c,v 1.10 2003/02/13 02:39:00 maf Exp $
+ *      $Id: flow-xlate.c,v 1.13 2003/11/25 05:14:42 maf Exp $
  */
 
 #include "ftconfig.h"
-#include "ftlib.h"
+#include <ftlib.h>
 
 #include <sys/types.h>
 #include <unistd.h>
@@ -48,42 +48,23 @@
 int debug;
 void usage(void);
 
-#define XLATE_AS0                    0x1
-#define XLATE_TO_LEGACY_NETWORK      0x2
-#define XLATE_TO_FLOW_NETWORK        0x4
-#define XLATE_PRIVACY_MASK           0x8
-#define XLATE_SCALE_FLOWS            0x10
-#define XLATE_SCALE_OCTETS           0x20
-#define XLATE_SCALE_PACKETS          0x40
-#define XLATE_SCALE_DURATION         0x80
-#define XLATE_SCALE_ALL              (0x10 | 0x20 | 0x40 | 0x80)
-#define XLATE_VERSION                0x100
-#define XLATE_SRC_TAG                0x200
-#define XLATE_DST_TAG                0x400
-
-struct xlate {
-  struct ftipmask ftipmask;
-  u_int32 flags;
-  u_int32 scale;
-  u_int32 privacy_mask;
-  u_int16 as;
-  u_int32 src_tag_mask, dst_tag_mask;
-};
-
 int main(int argc, char **argv)
 {
   struct ftio ftio_in, ftio_out;
   struct ftprof ftp;
+  struct ftxlate ftxlate;
+  struct ftxlate_def *ftfd;
   struct ftver ftv_in, ftv_out;
   struct ftset ftset;
-  struct fts3rec_all cur;
-  struct xlate xlate;
   struct fts3rec_offsets fo;
   char *rec, out_rec[FT_IO_MAXREC];
+  char *fname, *dname;
   unsigned int v1, v2;
-  int i, n, keep_input_time;
   u_int32 total_flows, cap_start, cap_end;
   u_int32 time_start, time_end;
+  int i, n, keep_input_time;
+  int xlate_version;
+  int no_config;
 
   /* init fterr */
   fterr_setid(argv[0]);
@@ -91,25 +72,21 @@ int main(int argc, char **argv)
   bzero(&ftv_in, sizeof ftv_in);
   bzero(&ftv_out, sizeof ftv_out);
   total_flows = 0;
-  xlate.flags = 0;
 
-  /* profile */
-  ftprof_start (&ftp);
+  fname = FT_PATH_CFG_XLATE;
+  dname = "default";
+
+  keep_input_time = 0;
+  xlate_version = 0; /* no */
+  no_config = 0; /* no */
 
   /* defaults + no compression */
   ftset_init(&ftset, 0);
 
-  keep_input_time = 0;
-
-  while ((i = getopt(argc, argv, "0:b:C:d:fh?klm:s:t:T:V:z:")) != -1)
+  while ((i = getopt(argc, argv, "b:C:d:h?knV:x:X:z:")) != -1)
 
     switch (i) {
 
-    case '0': /* AS */
-      xlate.flags |= XLATE_AS0;
-      xlate.as = atoi(optarg);
-      break;
-
     case 'b': /* output byte order */
       if (!strcasecmp(optarg, "little"))
         ftset.byte_order = FT_HEADER_LITTLE_ENDIAN;
@@ -127,37 +104,12 @@ int main(int argc, char **argv)
       debug = atoi(optarg);
       break;
 
-    case 'f': /* flow netmask */
-      xlate.flags |= XLATE_TO_FLOW_NETWORK;
-      break;
-
     case 'k': /* keep the start/end time from the input */
       keep_input_time = 1;
       break;
 
-    case 'l': /* legacy netmask */
-      xlate.flags |= XLATE_TO_LEGACY_NETWORK;
-      break;
-
-    case 'm': /* privacy_mask */
-      xlate.privacy_mask = scan_ip(optarg);
-      xlate.flags |= XLATE_PRIVACY_MASK;
-      break;
-
-    case 's': /* scale factor */
-      xlate.flags |= XLATE_SCALE_FLOWS | XLATE_SCALE_OCTETS |
-        XLATE_SCALE_PACKETS | XLATE_SCALE_DURATION;
-      xlate.scale = atoi(optarg);
-      break;
-
-    case 't': /* source tag */
-      xlate.flags |= XLATE_SRC_TAG;
-      xlate.src_tag_mask = strtoul(optarg, (char **)0L, 0);
-      break;
-
-    case 'T': /* destination tag */
-      xlate.flags |= XLATE_DST_TAG;
-      xlate.dst_tag_mask = strtoul(optarg, (char **)0L, 0);
+    case 'n': /* no config */
+      no_config = 1;
       break;
 
     case 'V': /* version */
@@ -175,7 +127,15 @@ int main(int argc, char **argv)
       } else {
         fterr_errx(1, "Version scan failed");
       }
-      xlate.flags |= XLATE_VERSION;
+      xlate_version = 1;
+      break;
+
+    case 'x': /* xlate file name */
+      fname = optarg;
+      break;
+
+    case 'X': /* xlate definition name */
+      dname = optarg;
       break;
 
     case 'z': /* compress level */
@@ -196,10 +156,18 @@ int main(int argc, char **argv)
   if (argc - optind)
     fterr_errx(1, "Extra arguments starting with %s.", argv[optind]);
 
-  if (xlate.flags & XLATE_PRIVACY_MASK)
-    /* setup for ftrec_mask_ip */
-    ftrec_compute_mask(&xlate.ftipmask, xlate.privacy_mask, xlate.privacy_mask,
-      ftset.byte_order);
+
+  /* may not need a config file */
+  if (!no_config) {
+
+    /* initialize and load translations */
+    if (ftxlate_load(&ftxlate, fname))
+      fterr_errx(1, "ftxlate_load(): failed");
+
+    if (!(ftfd = ftxlate_def_find(&ftxlate, dname))) 
+      fterr_errx(1, "ftxlate_def_find(): failed");
+
+  }
 
   /* input is stdin */
   if (ftio_init(&ftio_in, 0, FT_IO_FLAG_READ) < 0)
@@ -211,22 +179,26 @@ int main(int argc, char **argv)
     fterr_errx(1, "ftio_init(): failed");
 
   /* preserve start/end time from input stream? */
-  if (keep_input_time) {
-
+  if (keep_input_time) {      
+  
     time_start = ftio_get_cap_start(&ftio_in);
     time_end = ftio_get_cap_end(&ftio_in);
-    
+      
     if (time_start && time_end) {
-
+  
       ftio_set_preloaded(&ftio_out, 1);
       ftio_set_cap_time(&ftio_out, time_start, time_end);
-
-    } 
   
+    }
+
   } /* keep_input_time */
 
   ftio_get_ver(&ftio_in, &ftv_in);
 
+  if (!no_config)
+    if (ftxlate_def_test_xfields(ftfd, ftrec_xfield(&ftv_in)))
+      fterr_errx(1, "Xlate references a field not in flow.");
+
   ftv_in.s_version = FT_IO_SVERSION;
 
   if (!ftv_out.set)
@@ -247,120 +219,20 @@ int main(int argc, char **argv)
   if (ftio_write_header(&ftio_out) < 0)
     fterr_errx(1, "ftio_write_header(): failed");
 
-  /* check sanity */
-  if (xlate.flags & (XLATE_SRC_TAG)) {
-    if (ftio_check_xfield(&ftio_in, FT_XFIELD_SRC_TAG))
-      fterr_errx(1, "Flow record missing required field for format.");
-  }
-
-  if (xlate.flags & (XLATE_DST_TAG)) {
-    if (ftio_check_xfield(&ftio_in, FT_XFIELD_DST_TAG))
-      fterr_errx(1, "Flow record missing required field for format.");
-  }
-
-  if (xlate.flags & XLATE_TO_FLOW_NETWORK) {
-    if (ftio_check_xfield(&ftio_in, FT_XFIELD_SRCADDR | FT_XFIELD_DSTADDR
-      | FT_XFIELD_SRC_MASK | FT_XFIELD_DST_MASK))
-      fterr_errx(1, "Flow record missing required field for format.");
-  }
-
-  if (xlate.flags & XLATE_AS0) {
-    if (ftio_check_xfield(&ftio_in, FT_XFIELD_SRC_AS | FT_XFIELD_DST_AS))
-      fterr_errx(1, "Flow record missing required field for format.");
-  }
-
-  if (xlate.flags & XLATE_PRIVACY_MASK) {
-    if (ftio_check_xfield(&ftio_in, FT_XFIELD_SRCADDR | FT_XFIELD_DSTADDR))
-      fterr_errx(1, "Flow record missing required field for format.");
-  }
-
-  if (xlate.flags & XLATE_SCALE_OCTETS) {
-    if (ftio_check_xfield(&ftio_in, FT_XFIELD_DOCTETS))
-      fterr_errx(1, "Flow record missing required field for format.");
-  }
-
-  if (xlate.flags & XLATE_SCALE_PACKETS) {
-    if (ftio_check_xfield(&ftio_in, FT_XFIELD_DPKTS))
-      fterr_errx(1, "Flow record missing required field for format.");
-  }
-
   fts3rec_compute_offsets(&fo, &ftv_in);
 
+  /* profile */
+  ftprof_start (&ftp);
+
   while ((rec = ftio_read(&ftio_in))) {
 
     ++total_flows;
 
-    if (xlate.flags & XLATE_TO_LEGACY_NETWORK) {
+    if (!no_config)
+      if (ftxlate_def_eval(ftfd, rec, &fo) == FT_FIL_MODE_DENY)
+        continue;
 
-      cur.dstaddr = ((u_int32*)(rec+fo.dstaddr));
-      cur.srcaddr = ((u_int32*)(rec+fo.srcaddr));
-
-      if ((*cur.srcaddr & 0x80000000) == 0)
-        *cur.srcaddr &= 0xFF000000;
-      else if ((*cur.srcaddr & 0xC0000000) == 0x80000000)
-        *cur.srcaddr &= 0xFFFF0000;
-      else if ((*cur.srcaddr & 0xC0000000) == 0xC0000000)
-        *cur.srcaddr &= 0xFFFFFF00;
-
-      if ((*cur.dstaddr & 0x80000000) == 0)
-        *cur.dstaddr &= 0xFF000000;
-      else if ((*cur.dstaddr & 0xC0000000) == 0x80000000)
-        *cur.dstaddr &= 0xFFFF0000;
-      else if ((*cur.dstaddr & 0xC0000000) == 0xC0000000)
-        *cur.dstaddr &= 0xFFFFFF00;
-
-    }
-
-    if (xlate.flags & XLATE_TO_FLOW_NETWORK) {
-
-      cur.dstaddr = ((u_int32*)(rec+fo.dstaddr));
-      cur.srcaddr = ((u_int32*)(rec+fo.srcaddr));
-      cur.src_mask = ((u_int8*)(rec+fo.src_mask));
-      cur.dst_mask = ((u_int8*)(rec+fo.dst_mask));
-
-      *cur.srcaddr &= ipv4_len2mask(*cur.src_mask);
-      *cur.dstaddr &= ipv4_len2mask(*cur.dst_mask);
-
-    }
-
-    if (xlate.flags & XLATE_AS0) {
-
-      cur.src_as = ((u_int16*)(rec+fo.src_as));
-      cur.dst_as = ((u_int16*)(rec+fo.dst_as));
-
-      if (*cur.src_as == 0)
-        *cur.src_as = xlate.as;
-
-      if (*cur.dst_as == 0)
-        *cur.dst_as = xlate.as;
-
-    }
-
-    if (xlate.flags & XLATE_PRIVACY_MASK)
-      ftrec_mask_ip(rec, &ftv_in, &xlate.ftipmask);
-
-/* XXX split flow if overflow */
-    if (xlate.flags & XLATE_SCALE_OCTETS) {
-      cur.dOctets = ((u_int32*)(rec+fo.dOctets));
-      *cur.dOctets *= xlate.scale;
-    }
-
-    if (xlate.flags & XLATE_SCALE_PACKETS) {
-      cur.dPkts = ((u_int32*)(rec+fo.dPkts));
-      *cur.dPkts *= xlate.scale;
-    }
-
-    if (xlate.flags & XLATE_SRC_TAG) {
-      cur.src_tag = ((u_int32*)(rec+fo.src_tag));
-      *cur.src_tag &= xlate.src_tag_mask;
-    }
-
-    if (xlate.flags & XLATE_DST_TAG) {
-      cur.dst_tag = ((u_int32*)(rec+fo.dst_tag));
-      *cur.dst_tag &= xlate.dst_tag_mask;
-    }
-
-    if (xlate.flags & XLATE_VERSION) {
+    if (xlate_version) {
 
       ftrec_xlate(rec, &ftv_in, &out_rec, &ftv_out);
 
@@ -376,13 +248,15 @@ int main(int argc, char **argv)
 
   } /* while */
 
-
   if (ftio_close(&ftio_out) < 0)
     fterr_errx(1, "ftio_close(): failed");
 
   if (ftio_close(&ftio_in) < 0)
     fterr_errx(1, "ftio_close(): failed");
 
+  if (!no_config)
+    ftxlate_free(&ftxlate);
+
   if (debug > 0) {
     ftprof_end (&ftp, total_flows);
     ftprof_print(&ftp, argv[0], stderr);
@@ -393,11 +267,9 @@ int main(int argc, char **argv)
 } /* main */
 
 void usage(void) {
-
-  fprintf(stderr, "Usage: flow-xlate [-fhkl] [-0 AS0_substitution] [-b big|little] [-C comment]\n");
-  fprintf(stderr, "       [-d debug_level] [-m privacy_mask] [-s scale] [-t src_tag_mask]\n");
-  fprintf(stderr, "       [-T dst_tag_mask] [-V pdu_version] [-z z_level]\n");
+  fprintf(stderr, "Usage: flow-xlate [hkn] [-b big|little] [-C comment]\n");
+  fprintf(stderr, "       [-d debug_level] [-x xlate_fname] [-X xlate_definition]\n");
+  fprintf(stderr, "       [-V version] [-z z_level]\n");
   fprintf(stderr, "\n%s version %s: built by %s\n", PACKAGE, VERSION, FT_PROG_BUILD);
-
 }
 
